import 'zapt_sdk_flutter_platform_interface.dart';
import 'package:flutter/material.dart';
import 'package:flutter/foundation.dart';
import 'dart:async';
import 'dart:io';
import 'dart:convert';

import 'package:flutter/services.dart';
import 'package:webview_flutter/webview_flutter.dart';

class ZaptSdkFlutter {
  Future<String?> getMapLink(Map<String, dynamic> arguments) {
    return ZaptSdkFlutterPlatform.instance.getMapLink(arguments);
  }

  Future<void> requestPermissions() {
    return ZaptSdkFlutterPlatform.instance.requestPermissions();
  }

  Future<void> requestPermissionsBackground(Map<String, String> arguments) {
    return ZaptSdkFlutterPlatform.instance
        .requestPermissionsBackground(arguments);
  }

  Future<void> initialize(String placeId) {
    return ZaptSdkFlutterPlatform.instance.initialize(placeId);
  }

  Future<void> setDisableSyncingForPositioning(bool disable) {
    return ZaptSdkFlutterPlatform.instance
        .setDisableSyncingForPositioning(disable);
  }

  Future<void> setDisableSyncingForAnalytics(bool disable) {
    return ZaptSdkFlutterPlatform.instance
        .setDisableSyncingForAnalytics(disable);
  }

  Future<void> addListener(String eventName) {
    return ZaptSdkFlutterPlatform.instance.addListener(eventName);
  }

  Future<void> removeLocationListener() {
    return ZaptSdkFlutterPlatform.instance.removeLocationListener();
  }
}

class ZaptMap extends StatefulWidget {
  final String placeId;
  final Map<String, String>? options;
  const ZaptMap({Key? key, required this.placeId, this.options})
      : super(key: key);

  @override
  State<ZaptMap> createState() => _ZaptMapState();
}

class _ZaptMapState extends State<ZaptMap> {
  final _zaptSdkFlutterPlugin = ZaptSdkFlutter();
  WebViewController? controller;
  String mapLink = '';
  static const stream = EventChannel("ReactNativeZaptSdkBeaconsFound");

  @override
  void initState() {
    super.initState();
    initializeMap();
    if (Platform.isAndroid) WebView.platform = AndroidWebView();
  }

  @override
  void didUpdateWidget(ZaptMap oldWidget) {
    super.didUpdateWidget(oldWidget);
    if (oldWidget.placeId != widget.placeId ||
        !mapEquals(widget.options, oldWidget.options)) {
      initializeMap();
    }
  }

  Future<void> initializeMap() async {
    String mapLink;
    try {
      await _zaptSdkFlutterPlugin.initialize(widget.placeId);
      mapLink = await _zaptSdkFlutterPlugin.getMapLink(
              {'placeId': widget.placeId, 'options': widget.options}) ??
          "";
    } on PlatformException {
      mapLink = '';
    }
    if (mapLink.isNotEmpty) {
      controller?.loadUrl(mapLink);
    }
  }

  Future<void> initListemBeacons() async {
    await _zaptSdkFlutterPlugin.addListener("ReactNativeZaptSdkBeaconsFound");
    stream.receiveBroadcastStream().listen(_updateBeacons);
  }

  void _updateBeacons(beacons) {
    if (beacons.isNotEmpty) {
      String beaconsJSON = jsonEncode(beacons);
      controller?.runJavascript('window.onBeaconsFound($beaconsJSON);');
    }
  }

  @override
  Widget build(BuildContext context) {
    return SafeArea(
      child: WebView(
        javascriptMode: JavascriptMode.unrestricted,
        onWebViewCreated: (WebViewController webViewController) {
          controller = webViewController;
        },
        gestureNavigationEnabled: true,
        onPageFinished: (String url) {
          initListemBeacons();
        },
      ),
    );
  }
}
